
/* darkstat: a network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* Network [ab]use graph code */

#include "graph.h"
#include <assert.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>

pthread_mutex_t graph_mutex;

int64 graph_sec_in[GRAPH_SECS], graph_sec_out[GRAPH_SECS];
int64 graph_min_in[GRAPH_MINS], graph_min_out[GRAPH_MINS];
int64 graph_hr_in[GRAPH_HRS], graph_hr_out[GRAPH_HRS];
int64 graph_day_in[GRAPH_DAYS], graph_day_out[GRAPH_DAYS];

byte pos_sec = 0, pos_min = 0, pos_hr = 0, pos_day = 0;
byte g_secs = 0, g_mins = 0, g_hrs = 0;
dword last_time = 0;



void init_graph(void)
{
	int i;
	int64 zero;
	SET64(zero, 0,0);

	for (i=0; i<GRAPH_SECS; i++)
		graph_sec_in[i] = graph_sec_out[i] = zero;
	for (i=0; i<GRAPH_MINS; i++)
		graph_min_in[i] = graph_min_out[i] = zero;
	for (i=0; i<GRAPH_HRS; i++)
		graph_hr_in[i] = graph_hr_out[i] = zero;
	for (i=0; i<GRAPH_DAYS; i++)
		graph_day_in[i] = graph_day_out[i] = zero;
}



inline void graph_add_in(const dword amount)
{
	i64add32(graph_sec_in[pos_sec], amount);
	i64add32(graph_min_in[pos_min], amount);
	i64add32(graph_hr_in[pos_hr], amount);
	i64add32(graph_day_in[pos_day], amount);
}

inline void graph_add_out(const dword amount)
{
	i64add32(graph_sec_out[pos_sec], amount);
	i64add32(graph_min_out[pos_min], amount);
	i64add32(graph_hr_out[pos_hr], amount);
	i64add32(graph_day_out[pos_day], amount);
}



inline void graph_pushone(void)
{
	int64 zero;
	SET64(zero, 0,0);

	pos_sec = (pos_sec + 1) % GRAPH_SECS;
	graph_sec_in[pos_sec] = graph_sec_out[pos_sec] = zero;

	g_secs++;
	if (g_secs == 60) /* this is a constant */
	{
		/* there are exactly 60 seconds in a minute, how many
		 * seconds you actually want to graph is your problem
		 */
		g_secs = 0;
		pos_min = (pos_min + 1) % GRAPH_MINS;
		graph_min_in[pos_min] = graph_min_out[pos_min] = zero;

		g_mins++;
		if (g_mins == 60) /* constant */
		{
			g_mins = 0;
			pos_hr = (pos_hr + 1) % GRAPH_HRS;
			graph_hr_in[pos_hr] = graph_hr_out[pos_hr] = zero;

			g_hrs++;
			if (g_hrs == 24) /* constant */
			{
				FILE *fp = fopen(daylog_file, "a");
				struct tm *tm;
				time_t tarm;

				tarm = time(NULL);
				tm = localtime(&tarm);

#ifdef HAVE_64PRINT
			fprintf(fp, "%d/%d/%d|"
				"%" LONGLONG_FORMAT "u|"
				"%" LONGLONG_FORMAT "u\n",
					tm->tm_year+1900, tm->tm_mon+1,
					tm->tm_mday,
					graph_day_in[pos_day],
					graph_day_out[pos_day]);
#else
			{
				char *t1, *t2;
				
				fprintf(fp, "%d/%d/%d|%s|%s\n",
					tm->tm_year+1900, tm->tm_mon+1,
					tm->tm_mday,
					t1=strint64(graph_day_in[pos_day]),
					t2=strint64(graph_day_out[pos_day])
				       );

				free(t1);
				free(t2);
			}
#endif
				fclose(fp);

				g_hrs = 0;
				pos_day = (pos_day + 1) % GRAPH_DAYS;
				graph_day_in[pos_day] =
					graph_day_out[pos_day] = zero;
			}
		}
	}
}



/* argument is ignored */
inline void graph_rotate(dword now)
{
	if (now == last_time) return;
	if (last_time == 0)
	{
		last_time = now;
		return;
	}

	for (; last_time<now; last_time++)
		graph_pushone();
}



inline void graph_save_one(const byte pos, const byte GRAPH_SIZE,
		const byte g, const int64 graph_in[],
		const int64 graph_out[], FILE *fp)
{
	int i;

	fwrite(&pos, sizeof(pos), 1, fp);
	fwrite(&GRAPH_SIZE, sizeof(GRAPH_SIZE), 1, fp);
	fwrite(&g, sizeof(g), 1, fp);

	for (i=0; i<GRAPH_SIZE; i++)
	{
		fwrite64(graph_in[i], fp);
		fwrite64(graph_out[i], fp);
	}
}



void graph_save(FILE *fp)
{
	graph_save_one(pos_sec, GRAPH_SECS,
			g_secs, graph_sec_in, graph_sec_out, fp);

	graph_save_one(pos_min, GRAPH_MINS,
			g_mins, graph_min_in, graph_min_out, fp);

	graph_save_one(pos_hr , GRAPH_HRS,
			g_hrs , graph_hr_in , graph_hr_out , fp);

	graph_save_one(pos_day, GRAPH_DAYS,
			0     , graph_day_in, graph_day_out, fp);
}



inline int graph_load_one(byte *pos, byte GRAPH_SIZE, byte *g,
		int64 graph_in[], int64 graph_out[], FILE *fp)
{
	int read, i;
	byte g_size;

	read = fread(pos, sizeof(byte), 1, fp);
	if (!read) return 0;

	read = fread(&g_size, sizeof(byte), 1, fp);
	if (!read) return 0;
	if (g_size != GRAPH_SIZE)
	{
		fprintf(stderr, "GRAPH_SIZE compiled as %d, in DB it's %d\n",
				GRAPH_SIZE, g_size);
		return 0;
	}

	read = fread(g, sizeof(byte), 1, fp);
	if (!read) return 0;

	for (i=0; i<GRAPH_SIZE; i++)
	{
		fread64(graph_in[i], fp, read);
		if (!read) return 0;

		fread64(graph_out[i], fp, read);
		if (!read) return 0;
	}

	return 1;
}



int graph_load(FILE *fp)
{
	byte dummy;

	if (!graph_load_one(&pos_sec, GRAPH_SECS, &g_secs,
			graph_sec_in, graph_sec_out, fp)) return 0;

	if (!graph_load_one(&pos_min, GRAPH_MINS, &g_mins,
			graph_min_in, graph_min_out, fp)) return 0;

	if (!graph_load_one(&pos_hr , GRAPH_HRS, &g_hrs ,
			graph_hr_in , graph_hr_out , fp)) return 0;

	if (!graph_load_one(&pos_day, GRAPH_DAYS, &dummy ,
			graph_day_in, graph_day_out, fp)) return 0;

	return 1;
}

